/*
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *    https://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 */
package grails.gorm.tests.inheritance

import grails.gorm.annotation.Entity
import org.apache.grails.data.hibernate5.core.GrailsDataHibernate5TckManager
import org.apache.grails.data.testing.tck.base.GrailsDataTckSpec

class SubclassToOneProxySpec extends GrailsDataTckSpec<GrailsDataHibernate5TckManager> {
    void setupSpec() {
        manager.domainClasses.addAll([SuperclassProxy, SubclassProxy, HasOneProxy])
    }

    void "the hasOne is a proxy and unwraps"() {
        given:
        SubclassProxy dog = new SubclassProxy().save()
        new HasOneProxy(superclassProxy: dog).save()
        manager.session.flush()
        manager.session.clear()
        HasOneProxy owner = HasOneProxy.first()

        expect:
        manager.session.mappingContext.proxyFactory.isProxy(owner.@superclassProxy)
    }
}

@Entity
class SuperclassProxy {
}

@Entity
class SubclassProxy extends SuperclassProxy {
}

@Entity
class HasOneProxy {
    SuperclassProxy superclassProxy
}
