/*
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *    https://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 */

package grails.web.mapping

import groovy.transform.CompileStatic
import org.grails.web.mapping.DefaultUrlMappingEvaluator
import org.grails.web.mapping.DefaultUrlMappingsHolder
import org.springframework.context.ApplicationContext
import org.springframework.context.ApplicationContextAware

/**
 * Helper class for creating a {@link UrlMapping}. Useful for testing
 *
 * @author Graeme Rocher
     * @since 3.1
 */
@CompileStatic
class UrlMappingsFactory implements ApplicationContextAware {

    ApplicationContext applicationContext

    UrlMappings create(Closure mappingsClosure) {
        def evaluator = new DefaultUrlMappingEvaluator(applicationContext)
        return new DefaultUrlMappingsHolder(evaluator.evaluateMappings(mappingsClosure))
    }

    UrlMappings create(Class mappingsClass) {
        def evaluator = new DefaultUrlMappingEvaluator(applicationContext)
        return new DefaultUrlMappingsHolder(evaluator.evaluateMappings(mappingsClass))
    }
}
